#include<avr/io.h>
#include"MMC_SD/MMC_SD.h"
#include"FAT/FAT.h"
#include <util/delay.h>

// all on PORTC
#define	OLED_CD	  PC7   
#define	OLED_WR   PC6	// WR in 8080 mode
#define	OLED_RD   PC5	// RD in 8080 mode
#define	OLED_CS	  PC4
#define	OLED_RES	PC2

#define CLR_CD PORTC &= ~(1<<OLED_CD);
#define SET_CD PORTC |=  (1<<OLED_CD);

#define CLR_CS PORTC &= ~(1<<OLED_CS);
#define SET_CS PORTC |=  (1<<OLED_CS);

#define CLR_RESET PORTC &= ~(1<<OLED_RES);
#define SET_RESET PORTC |=  (1<<OLED_RES);

// 8080 mode pin functions
#define CLR_WR PORTC &= ~(1 << OLED_WR);   // 8080 mode
#define SET_WR PORTC |=  (1 << OLED_WR);   // 8080 mode
#define CLR_RD PORTC &= ~(1 << OLED_RD);   // 8080 mode
#define SET_RD PORTC |=  (1 << OLED_RD);   // 8080 mode

// color definitions
#define	BLACK	  0x0000
#define	BLUE	  0x001F
#define	RED 	  0xF800
#define	GREEN   0x07E0
#define CYAN	  0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE	  0xFFFF

#define uint8 unsigned char
#define uint16 unsigned int
#define uint32 unsigned long

unsigned char * PATH = "\\batch";

extern uint16 SectorsPerClust;
extern uint16 FirstDataSector; 
extern uint8 FAT32_Enable;

struct FileInfoStruct FileInfo;

struct direntry PictureInfo;
uint16 totalfiles = 0;

/*************************************************/
void delay(unsigned int t)
{
	unsigned char t1;
	while(t--)
	for(t1=11;t1>0;t1--)
	{asm("nop");}
}

#define uchar unsigned char
#define uint unsigned int

//-----------------------------------------------------------------------

void SendCommand(unsigned int command)
{
	CLR_CS;
	CLR_CD;
	SET_RD;
	SET_WR;

	PORTA=command;

	CLR_WR;
	SET_WR;
	SET_CS;
}

void WriteData(unsigned int data)
{
	CLR_CS;
	SET_CD;
	SET_RD;
	SET_WR;

	PORTA=data;

	CLR_WR;
	SET_WR;
	SET_CS;
}

/*************************************************/
void initialization()
{
	SET_RD;
	SET_WR;
	SET_CS;
	SET_CD;
	PORTA=0x00;
	PORTE=0x00;

	CLR_RESET;
	delay(200);
	SET_RESET;
	delay(500);


  SendCommand(0x61);        //scan driver control,comand explan at page 34
  WriteData(0x00);

  SendCommand(0x60);        //scan voltage setting,comand explan at page 34
  WriteData(0x00);

  SendCommand(0x01);   	    //command explain at page 12
  WriteData(0x03);          //address increment direction(x:0->159,y:0->128,x direction increment at first then y direction)
        										//8bit interface with 2 transmissions 65k colour

  SendCommand(0x06);   	    //command explain at page 17
  WriteData(0x00);          //X address

  SendCommand(0x07);   	    //command explain at page 17
  WriteData(0x00);          //Y address

  SendCommand(0x02);   	    //command explain at page 13
  WriteData(0x03);          //display size:160*128 , display frame :90Hz

  SendCommand(0x43);   	    //command is not explain at page 
  WriteData(0x00);          //enable gamma

  SendCommand(0x04);   	    //command explain at page 14
  WriteData(0x00);          //no display rotate   160x128    

  SendCommand(0x03);   	    //command explain at page 13
  WriteData(0x01);          //scan direction:0->159,scan seq direction:S0 S1 S2.....S127

  SendCommand(0x27);   	    //command explain at page 28
  WriteData(0x01);          //128 scan line time

  SendCommand(0x20);   	    //command explain at page 27
  WriteData(0x06);          //set RED disprecharge time of scan line

  SendCommand(0x21);   	    //command explain at page 27
  WriteData(0x04);          //set GREEN disprecharge time of scan line

  SendCommand(0x22);   	    //command explain at page 27
  WriteData(0x08);          //set BLUE disprecharge time of scan line

  SendCommand(0x23);   	    //command explain at page 28
  WriteData(0x03);          //set RED peak time of scan line

  SendCommand(0x24);   	    //command explain at page 28
  WriteData(0x04);          //set GREEN peak time of scan line

  SendCommand(0x25);   	    //command explain at page 28
  WriteData(0x04);          //set BLUE peak time of scan line

  SendCommand(0x50);          //red dot current set, command explan at page30
  WriteData(0x64); 

  SendCommand(0x51);          //green dot current set, command explan at page30 
  WriteData(0x62); 

  SendCommand(0x52);          //blue dot current set, command explan at page31 
  WriteData(0x62); 

  SendCommand(0x53);            //red peak current set,command explan at page 31 
  WriteData(0x03);    

  SendCommand(0x54);            //green peak current set,command explan at page 32
  WriteData(0x03);     

  SendCommand(0x55);            //blue peak current set,command explan at page 32
  WriteData(0x03);    

  SendCommand(0x26);   	    //command explain at page 28
  WriteData(0x06);          //period in scan to scan 0~63

  SendCommand(0x5a);             //data gray level control set,comand explan at page 33
  WriteData(0x01);

  SendCommand(0xa0);               //power save set,comand explan at page 35
  WriteData(0x00);               //0 go   1 stop

  SendCommand(0x00);   	    //this command must be again at last ,command explain at page 11
  WriteData(0x80);          //dispaly off,set direction of data is nomall direction,RGB,RESET=1


  //***************************************
  SendCommand(0x40);   	    //command explain at page 29
  WriteData(0x00);          //Red Lookup Set.
  WriteData(0x00);
  WriteData(0x00);
  WriteData(0x01);
  WriteData(0x02);          //5

  WriteData(0x03);
  WriteData(0x04);
  WriteData(0x05);
  WriteData(0x06);
  WriteData(0x07);          //10

  WriteData(0x08);
  WriteData(0x09);
  WriteData(0x0a);
  WriteData(0x0b);
  WriteData(0x0c);          //15

  WriteData(0x0d);
  WriteData(0x0e);
  WriteData(0x0f);
  WriteData(0x10);
  WriteData(0x12);          //20

  WriteData(0x13);
  WriteData(0x14);
  WriteData(0x16);
  WriteData(0x17);
  WriteData(0x19);          //25

  WriteData(0x1b);
  WriteData(0x1c);
  WriteData(0x1e);
  WriteData(0x20);
  WriteData(0x22);          //30

  WriteData(0x24);
  WriteData(0x26);
  WriteData(0x28);
  WriteData(0x2a);
  WriteData(0x2c);          //35

  WriteData(0x2f);
  WriteData(0x31);
  WriteData(0x33);
  WriteData(0x36);
  WriteData(0x38);          //40

  WriteData(0x3b);
  WriteData(0x3d);
  WriteData(0x40);
  WriteData(0x43);
  WriteData(0x45);          //45

  WriteData(0x48);
  WriteData(0x4b);
  WriteData(0x4e);
  WriteData(0x51);
  WriteData(0x53);          //50

  WriteData(0x56);
  WriteData(0x59);
  WriteData(0x5c);
  WriteData(0x60);
  WriteData(0x63);          //55

  WriteData(0x66);
  WriteData(0x69);
  WriteData(0x6c);
  WriteData(0x70);
  WriteData(0x73);          //60

  WriteData(0x76);
  WriteData(0x79);
  WriteData(0x7c);
  WriteData(0x7f);          //64

  //*************************************
  SendCommand(0x41);   	    //command explain at page 30
  WriteData(0x00);          //Green Lookup Set.
  WriteData(0x00);
  WriteData(0x00);
  WriteData(0x01);
  WriteData(0x02);          //5

  WriteData(0x03);
  WriteData(0x04);
  WriteData(0x05);
  WriteData(0x06);
  WriteData(0x07);          //10

  WriteData(0x08);
  WriteData(0x09);
  WriteData(0x0a);
  WriteData(0x0b);
  WriteData(0x0c);          //15

  WriteData(0x0d);
  WriteData(0x0e);
  WriteData(0x0f);
  WriteData(0x10);
  WriteData(0x12);          //20

  WriteData(0x13);
  WriteData(0x14);
  WriteData(0x16);
  WriteData(0x17);
  WriteData(0x19);          //25

  WriteData(0x1b);
  WriteData(0x1c);
  WriteData(0x1e);
  WriteData(0x20);
  WriteData(0x22);          //30

  WriteData(0x24);
  WriteData(0x26);
  WriteData(0x28);
  WriteData(0x2a);
  WriteData(0x2c);          //35

  WriteData(0x2f);
  WriteData(0x31);
  WriteData(0x33);
  WriteData(0x36);
  WriteData(0x38);          //40

  WriteData(0x3b);
  WriteData(0x3d);
  WriteData(0x40);
  WriteData(0x43);
  WriteData(0x45);          //45

  WriteData(0x48);
  WriteData(0x4b);
  WriteData(0x4e);
  WriteData(0x51);
  WriteData(0x53);          //50

  WriteData(0x56);
  WriteData(0x59);
  WriteData(0x5c);
  WriteData(0x60);
  WriteData(0x63);          //55

  WriteData(0x66);
  WriteData(0x69);
  WriteData(0x6c);
  WriteData(0x70);
  WriteData(0x73);          //60

  WriteData(0x76);
  WriteData(0x79);
  WriteData(0x7c);
  WriteData(0x7f);          //64
  //*************************************************
  SendCommand(0x42);   	    //command explain at page 30
  WriteData(0x00);          //Blue Lookup Set.
  WriteData(0x00);
  WriteData(0x00);
  WriteData(0x01);
  WriteData(0x02);          //5

  WriteData(0x03);
  WriteData(0x04);
  WriteData(0x05);
  WriteData(0x06);
  WriteData(0x07);          //10

  WriteData(0x08);
  WriteData(0x09);
  WriteData(0x0a);
  WriteData(0x0b);
  WriteData(0x0c);          //15

  WriteData(0x0d);
  WriteData(0x0e);
  WriteData(0x0f);
  WriteData(0x10);
  WriteData(0x12);          //20

  WriteData(0x13);
  WriteData(0x14);
  WriteData(0x16);
  WriteData(0x17);
  WriteData(0x19);          //25

  WriteData(0x1b);
  WriteData(0x1c);
  WriteData(0x1e);
  WriteData(0x20);
  WriteData(0x22);          //30

  WriteData(0x24);
  WriteData(0x26);
  WriteData(0x28);
  WriteData(0x2a);
  WriteData(0x2c);          //35

  WriteData(0x2f);
  WriteData(0x31);
  WriteData(0x33);
  WriteData(0x36);
  WriteData(0x38);          //40

  WriteData(0x3b);
  WriteData(0x3d);
  WriteData(0x40);
  WriteData(0x43);
  WriteData(0x45);          //45

  WriteData(0x48);
  WriteData(0x4b);
  WriteData(0x4e);
  WriteData(0x51);
  WriteData(0x53);          //50

  WriteData(0x56);
  WriteData(0x59);
  WriteData(0x5c);
  WriteData(0x60);
  WriteData(0x63);          //55

  WriteData(0x66);
  WriteData(0x69);
  WriteData(0x6c);
  WriteData(0x70);
  WriteData(0x73);          //60

  WriteData(0x76);
  WriteData(0x79);
  WriteData(0x7c);
  WriteData(0x7f);          //64
}

// **********************ȫ************************* /
void display_Color(unsigned int color)
{
  unsigned int k,i;
  SendCommand(0x06);   	    //command explain at page 17
  WriteData(0x00);          //X address

  SendCommand(0x07);   	    //command explain at page 17
  WriteData(0x00);          //Y address

  SendCommand(0x08);   	    //command explain at page 17
  WriteData(0x00);          //star address of X in the rectangle windows

  SendCommand(0x09);   	    //command explain at page 17
  WriteData(0x9f);          //end address of X in the rectangle windows

  SendCommand(0x0a);   	    //command explain at page 17
  WriteData(0x00);          //star address of Y in the rectangle windows

  SendCommand(0x0b);   	    //command explain at page 18
  WriteData(0x7f);          //end address of Y in the rectangle windows

  SendCommand(0x05);   	    //command explain at page 15

	for(k=0;k<160;k++)
	{
    for(i=0;i<128;i++)
		{    		
    	WriteData(color);
			WriteData(color>>8);
		}
	}
}


/*************************************************/
void Display_Home()
{
  SendCommand(0x06);   	    //command explain at page 17
  WriteData(0x00);          //X address

  SendCommand(0x07);   	    //command explain at page 17
  WriteData(0x00);          //Y address

  SendCommand(0x08);   	    //command explain at page 17
  WriteData(0x00);          //star address of X in the rectangle windows

  SendCommand(0x09);   	    //command explain at page 17
  WriteData(0x9f);          //end address of X in the rectangle windows

  SendCommand(0x0a);   	    //command explain at page 17
  WriteData(0x00);          //star address of Y in the rectangle windows

  SendCommand(0x0b);   	    //command explain at page 18
  WriteData(0x7f);          //end address of Y in the rectangle windows

  SendCommand(0x05);   	    //command explain at page 15
  }

void Picture_Slide_Show()
{
	uint8 i;              //loop variable
	uint16 j;             //loop variable
	uint32 p;             //cluster
	uint16 *buffer;				//buffer
	uint16 pics = 1;
	uint16 slide_show_flag = 1;
	uint8 sector;
	uint32 pixels;

	if (totalfiles == 0)
	return;

	buffer = malloc(512);
	if (buffer == 0)
	return;

next: 	//label for "goto"
	display_Color(BLACK);

	Search(PATH, &PictureInfo, &pics);                                         //find the file
	p = PictureInfo.deStartCluster + (((uint32)PictureInfo.deHighClust) << 16);//the first cluster of the file

	sector = 0;

	FAT_LoadPartCluster(p, sector, buffer);	//read a sector


	pixels = (uint32) 160 * 128; // total # of pixels to write

	Display_Home();

	j = 0; // byte count
	
	while (1)
	{
		WriteData(buffer[j]);  // write 8 bits
		WriteData(buffer[j]>>8);  // write 8 bits
    j++;
		pixels--;               // which is one pixel

		if (pixels == 0)        
		break;

		if (j == 256)           // time for a new sector     
		{
			sector++;
			if (sector == SectorsPerClust)
			{
				p = FAT_NextCluster(p);				// read next cluster
				sector = 0;
			}

			FAT_LoadPartCluster(p, sector, buffer);	// read a sector
			j = 0;
		}
	}

	if (slide_show_flag)
	{
		for (i=0;i<30;i++)	// delay for a while
		delay(0xffff);

		pics++;					// increment picture number
		if (pics > totalfiles)	// if last
		pics = 1;			// wrap around

		goto next;				// go show next pic
	}
	free(buffer);
}
/*************************************************/

int main()
{
	PORTD = 0xF8;	    // pull-up on switches, all others off
	DDRD  = 0x0F;   	// VPP and led pins output

	DDRA = 0xFF;    	// low data port, all outputs
	DDRE = 0xFF;    	// low data port, all outputs
	
	PORTC = 0xFE;	    // all pins high
	DDRC  = 0xFE;	    // all outputs

	uint8 retry = 0;

	OSCCAL = 0x00;    //in order to operate some low speed card the initialization should run at lowest speed

	delay(0xffff);

	MMC_SD_Init();    //SPI initialize

	// Initialize the display
	initialization();

	sd_restart: 	//label for "goto"

	while (MMC_SD_Reset())  // Initialize SD Card
	{
		retry++;
		if (retry > 254)
		{
			while (1)
			{
				goto sd_restart;				// try again
			}
		}
	}

	OSCCAL = 0xff;      //run at high speed for normal operation for SD Card

	delay(0xffff);      

	if (FAT_Init())   	//initialize file system - FAT16 and FAT32 are supported
	{
		while (1)
		{
			goto sd_restart;				// try again
		}
	}

	Search(PATH, &PictureInfo, &totalfiles);

	Picture_Slide_Show();

  while(1)
  {
    display_Color(RED);
	  delay(1000);
    display_Color(GREEN);
	  delay(1000);
    display_Color(BLUE);
	  delay(1000);   	
    display_Color(WHITE);   
	  delay(1000);
    display_Color(BLACK);   
	  delay(1000);
  }
	return 0;
}

